@@ -38,6 +38,11 @@ using namespace std;
 #include <stdlib.h>
 #include <pthread.h>
 
+#include <sys/select.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <unistd.h>
+
 char *strndup(const char *s, size_t n)
 {
   char *str;
@@ -97,7 +102,9 @@ CMSN::CMSN(CICQDaemon *_pDaemon, int _nPipe) : m_vlPacketBucket(211)
 
   // pthread stuff now
   pthread_mutex_init(&mutex_StartList, 0);
+  pthread_mutex_init(&mutex_MSNEventList, 0);
   pthread_mutex_init(&mutex_ServerSocket, 0);
+  pthread_mutex_init(&mutex_Bucket, 0);
 }
 
 CMSN::~CMSN()
@@ -125,36 +132,69 @@ CMSN::~CMSN()
 
 void CMSN::StorePacket(SBuffer *_pBuf, int _nSock)
 {
-  BufferList &b = m_vlPacketBucket[HashValue(_nSock)];
-  b.push_front(_pBuf);
+  if (_pBuf->m_bStored == false)
+  {
+    pthread_mutex_lock(&mutex_Bucket);
+    BufferList &b = m_vlPacketBucket[HashValue(_nSock)];
+    b.push_front(_pBuf);
+    pthread_mutex_unlock(&mutex_Bucket);
+  }
 }
 
-void CMSN::RemovePacket(string _strUser, int _nSock)
+void CMSN::RemovePacket(string _strUser, int _nSock, int nSize)
 {
+  pthread_mutex_lock(&mutex_Bucket);
   BufferList &b = m_vlPacketBucket[HashValue(_nSock)];
   BufferList::iterator it;
+  SBuffer *pNewBuf = 0;
+  int nNewSize = 0;
+
   for (it = b.begin(); it != b.end(); it++)
   {
     if ((*it)->m_strUser == _strUser)
     {
+      // Found a packet that has part of another packet at the end
+      // so we have to save it and put it back on the queue.
+      if (nSize)
+      {
+	nNewSize = (*it)->m_pBuf->getDataSize() - nSize;
+	if (nNewSize)
+	{
+	  pNewBuf = new SBuffer;
+	  pNewBuf->m_strUser = _strUser;
+	  pNewBuf->m_pBuf = new CMSNBuffer(nNewSize);
+	  pNewBuf->m_pBuf->Pack((*it)->m_pBuf->getDataStart()+nSize,
+				nNewSize);
+	  pNewBuf->m_bStored = true;
+	}			   
+      }
+      
       b.erase(it);
       break;
     }
   }
+
+  // Now add it here
+  if (pNewBuf)
+    b.push_front(pNewBuf);
+  pthread_mutex_unlock(&mutex_Bucket);
 }
 
 SBuffer *CMSN::RetrievePacket(const string &_strUser, int _nSock)
 {
+  pthread_mutex_lock(&mutex_Bucket);
   BufferList &b = m_vlPacketBucket[HashValue(_nSock)];
   BufferList::iterator it;
   for (it = b.begin(); it != b.end(); it++)
   {
     if ((*it)->m_strUser == _strUser)
     {
+      pthread_mutex_unlock(&mutex_Bucket);
       return *it;
     }
   }
-  
+  pthread_mutex_unlock(&mutex_Bucket);
+
   return 0;
 }
   
@@ -176,6 +216,131 @@ ICQEvent *CMSN::RetrieveEvent(unsigned long _nTag)
   return e;
 }
 
+void CMSN::HandlePacket(int _nSocket, CMSNBuffer &packet, const char* _szUser)
+{
+  SBuffer *pBuf = RetrievePacket(_szUser, _nSocket);
+  bool bProcess = false;
+
+  if (pBuf)
+    *(pBuf->m_pBuf) += packet;
+  else
+  {
+    pBuf = new SBuffer;
+    pBuf->m_pBuf = new CMSNBuffer(packet);
+    pBuf->m_strUser = _szUser;
+    pBuf->m_bStored = false;
+  }
+
+  do
+  {
+    char *szNeedle;
+    CMSNBuffer *pPart = 0;
+    unsigned long nFullSize = 0;
+    bProcess = false;
+
+    if ((szNeedle = strstr((char *)pBuf->m_pBuf->getDataStart(),
+                           "\r\n")))
+    {
+      bool isMSG = (memcmp(pBuf->m_pBuf->getDataStart(), "MSG", 3) == 0);
+
+      if (/*memcmp(pBuf->m_pBuf->getDataStart(), "MSG", 3) == 0*/ isMSG ||
+          memcmp(pBuf->m_pBuf->getDataStart(), "NOT", 3) == 0)
+      {
+        pBuf->m_pBuf->SkipParameter(); // MSG, NOT
+        if (isMSG)
+        {
+          pBuf->m_pBuf->SkipParameter(); // Hotmail
+          pBuf->m_pBuf->SkipParameter(); // Hotmail
+        }
+        string strSize = pBuf->m_pBuf->GetParameter();
+        int nSize = atoi(strSize.c_str());
+
+        // Cut the packet instead of passing it all along
+        // we might receive 1 full packet and part of another.
+        if (nSize <= (pBuf->m_pBuf->getDataPosWrite() - pBuf->m_pBuf->getDataPosRead()))
+        {
+          nFullSize = nSize + pBuf->m_pBuf->getDataPosRead() - pBuf->m_pBuf->getDataStart() + 1;
+          if (pBuf->m_pBuf->getDataSize() > nFullSize)
+          {
+            // Hack to save the part and delete the rest
+            if (pBuf->m_bStored == false)
+            {
+              StorePacket(pBuf, _nSocket);
+              pBuf->m_bStored = true;
+            }
+
+            // We have a packet, with part of another one at the end
+            pPart = new CMSNBuffer(nFullSize);
+            pPart->Pack(pBuf->m_pBuf->getDataStart(), nFullSize);
+          }
+          bProcess = true;
+        }
+      }
+      else //if(memcmp(pBuf->m_pBuf->getDataStart(), "ACK", 3) == 0)
+      {
+        int nSize = szNeedle - pBuf->m_pBuf->getDataStart() +2;
+
+        // Cut the packet instead of passing it all along
+        // we might receive 1 full packet and part of another.
+        if (nSize <= (pBuf->m_pBuf->getDataPosWrite() - pBuf->m_pBuf->getDataPosRead()))
+        {
+          nFullSize = nSize + pBuf->m_pBuf->getDataPosRead() - pBuf->m_pBuf->getDataStart();
+
+          if (pBuf->m_pBuf->getDataSize() > nFullSize)
+          {
+            // Hack to save the part and delete the rest
+            if (pBuf->m_bStored == false)
+            {
+              StorePacket(pBuf, _nSocket);
+              pBuf->m_bStored = true;
+            }
+
+            // We have a packet, with part of another one at the end
+            pPart = new CMSNBuffer(nFullSize);
+            pPart->Pack(pBuf->m_pBuf->getDataStart(), nFullSize);
+          }
+          bProcess = true;
+        }
+      }
+
+      if (!bProcess)
+      {
+        // Save it
+        StorePacket(pBuf, _nSocket);
+        pBuf->m_bStored = true;
+      }
+
+      pBuf->m_pBuf->Reset();
+    }
+    else
+    {
+      // Need to save it, doens't have much data yet
+      StorePacket(pBuf, _nSocket);
+      pBuf->m_bStored = true;
+      bProcess = false;
+    }
+
+    if (bProcess)
+    {
+      // Handle it, and then remove it from the queue
+      if (m_nServerSocket == _nSocket)
+        ProcessServerPacket(pPart ? pPart : pBuf->m_pBuf);
+      else
+        ProcessSBPacket(const_cast<char *>(_szUser), pPart ? pPart : pBuf->m_pBuf,
+                        _nSocket);
+      RemovePacket(_szUser, _nSocket, nFullSize);
+      if (pPart)
+        delete pPart;
+      else
+        delete pBuf;
+      pBuf = RetrievePacket(_szUser, _nSocket);
+    }
+    else
+      pBuf = 0;
+
+  } while (pBuf);
+}
+
 string CMSN::Decode(const string &strIn)
 {
   string strOut = "";
@@ -237,6 +402,7 @@ void CMSN::Run()
   while (!m_bExit)
   {
     pthread_mutex_lock(&mutex_ServerSocket);
+    FD_ZERO(&f);
     f = gSocketMan.SocketSet();
     nNumDesc = gSocketMan.LargestSocket() + 1;
  
@@ -280,12 +446,14 @@ void CMSN::Run()
             CMSNBuffer packet(sock->RecvBuffer());
             sock->ClearRecvBuffer();
             gSocketMan.DropSocket(sock);
-            ProcessServerPacket(packet);
+
+            HandlePacket(m_nServerSocket, packet, m_szUserName);
           }
           else
           {
             // Time to reconnect
             gLog.Info("%sDisconnected from server, reconnecting.\n", L_MSNxSTR);
+            sleep(1);
             int nSD = m_nServerSocket;
             m_nServerSocket = -1;
             gSocketMan.DropSocket(sock);
@@ -328,73 +496,23 @@ void CMSN::Run()
           if (sock && sock->RecvRaw())
           {
             CMSNBuffer packet(sock->RecvBuffer());
-            bool bProcess = false;
             sock->ClearRecvBuffer();
             char *szUser = strdup(sock->OwnerId());
             gSocketMan.DropSocket(sock);
+
+            HandlePacket(nCurrent, packet, szUser);
             
-            // Build the packet with last received portion
-            string strUser(szUser);
-            SBuffer *pBuf = RetrievePacket(strUser, nCurrent);
-            if (pBuf)
-            {
-              *(pBuf->m_pBuf) += packet;
-            }
-            else
-            {
-              pBuf = new SBuffer;
-              pBuf->m_pBuf = new CMSNBuffer(packet);
-              pBuf->m_strUser = strUser;
-            }
-            
-            // Do we have the entire packet?
-            char *szNeedle;
-            if ((szNeedle = strstr((char *)pBuf->m_pBuf->getDataStart(), "\r\n")))
-            {
-              // We have a basic packet, now check for a packet that has a payload
-              if (memcmp(pBuf->m_pBuf->getDataStart(), "MSG", 3) == 0)
-              {
-                pBuf->m_pBuf->SkipParameter(); // command
-                pBuf->m_pBuf->SkipParameter(); // user id
-                pBuf->m_pBuf->SkipParameter(); // alias
-                string strSize = pBuf->m_pBuf->GetParameter();
-                int nSize = atoi(strSize.c_str());
-                  
-                // FIXME: Cut the packet instead of passing it all along
-                // we might receive 1 full packet and part of another.
-                if (nSize <= (pBuf->m_pBuf->getDataPosWrite() - pBuf->m_pBuf->getDataPosRead()))
-                {
-                  bProcess = true;
-                }
-                else
-                {
-                  StorePacket(pBuf, nCurrent);
-                }
-                
-                pBuf->m_pBuf->Reset();  
-              }
-              else
-                bProcess = true; // no payload
-            }  
-            
-            if (bProcess)
-            {
-              ProcessSBPacket(szUser, pBuf->m_pBuf, sock->Descriptor());
-              RemovePacket(strUser, nCurrent);
-              delete pBuf;
-            }
-            
-            free(szUser);
-          }
-          else
-          {
-            // Shouldn't get here, as we close the socket with a BYE command.
-            // But just to be safe..
-            if (sock)
-              gSocketMan.DropSocket(sock);
-            gSocketMan.CloseSocket(nCurrent);
-          }
-        }
+	    free(szUser);
+	  }
+	  else
+	  {
+	    // Shouldn't get here, as we close the socket with a BYE command.
+	    // But just to be safe..
+	    if (sock)
+	      gSocketMan.DropSocket(sock);
+	    gSocketMan.CloseSocket(nCurrent);
+	  }
+	}
       }
 
       nCurrent++;
@@ -505,13 +623,13 @@ void CMSN::ProcessSignal(CSignal *s)
 
     case PROTOxSENDxREFUSExAUTH:
     {
-      CRefuseAuthSignal *sig = static_cast<CRefuseAuthSignal *>(s);
+//      CRefuseAuthSignal *sig = static_cast<CRefuseAuthSignal *>(s);
       break;
     }
 
     case PROTOxREQUESTxINFO:
     {
-      CRequestInfo *sig = static_cast<CRequestInfo *>(s);
+//      CRequestInfo *sig = static_cast<CRequestInfo *>(s);
       break;
     }
 
@@ -542,3 +660,83 @@ void CMSN::ProcessSignal(CSignal *s)
 
   delete s;
 }
+
+void CMSN::WaitDataEvent(CMSNDataEvent *_pEvent)
+{
+  pthread_mutex_lock(&mutex_MSNEventList);
+  m_lMSNEvents.push_back(_pEvent);
+  pthread_mutex_unlock(&mutex_MSNEventList);
+}
+
+bool CMSN::RemoveDataEvent(CMSNDataEvent *pData)
+{
+  list<CMSNDataEvent *>::iterator it;
+  pthread_mutex_lock(&mutex_MSNEventList);
+  for (it = m_lMSNEvents.begin(); it != m_lMSNEvents.end(); it++)
+  {
+    if ((*it)->getUser() == pData->getUser() &&
+	(*it)->getSocket() == pData->getSocket())
+    {
+      // Close the socket
+      gSocketMan.CloseSocket(pData->getSocket());
+
+      CConversation *pConv = m_pDaemon->FindConversation(pData->getSocket());
+      if (pConv)
+        m_pDaemon->RemoveConversation(pConv->CID());
+
+      m_lMSNEvents.erase(it);
+      delete pData;
+      pData = 0;
+      break;
+    }
+  }
+  pthread_mutex_unlock(&mutex_MSNEventList);
+
+  return (pData == 0);
+}
+
+CMSNDataEvent *CMSN::FetchDataEvent(const string &_strUser, int _nSocket)
+{
+  CMSNDataEvent *pReturn = 0;
+  list<CMSNDataEvent *>::iterator it;
+  pthread_mutex_lock(&mutex_MSNEventList);
+  for (it = m_lMSNEvents.begin(); it != m_lMSNEvents.end(); it++)
+  {
+    if ((*it)->getUser() == _strUser && (*it)->getSocket() == _nSocket)
+    {
+      pReturn = *it;
+      break;
+    }
+  }
+
+  if (!pReturn)
+  {
+    pReturn = FetchStartDataEvent(_strUser);
+    if (pReturn)
+      pReturn->setSocket(_nSocket);
+  }
+  pthread_mutex_unlock(&mutex_MSNEventList);
+
+  return pReturn;
+}
+
+CMSNDataEvent *CMSN::FetchStartDataEvent(const string &_strUser)
+{
+  CMSNDataEvent *pReturn = 0;
+  list<CMSNDataEvent *>::iterator it;
+  for (it = m_lMSNEvents.begin(); it != m_lMSNEvents.end(); it++)
+  {
+    if ((*it)->getUser() == _strUser && (*it)->getSocket() == -1)
+    {
+      pReturn = *it;
+      break;
+    }
+  }
+
+  return pReturn;  
+}
+
+void CMSN::PushPluginSignal(CICQSignal *p)
+{
+  m_pDaemon->PushPluginSignal(p);
+}
