@@ -1,19 +1,22 @@
 // -*- c-basic-offset: 2 -*-
 /*
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 1999-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 // written by Graham Roff <graham@licq.org>
 // KDE support by Dirk Mueller <dirk@licq.org>
@@ -49,6 +52,7 @@
 #include "mainwin.h"
 #include "licq_user.h"
 #include "ewidgets.h"
+#include "gui-strings.h"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -67,14 +71,16 @@
 /*
   Constructs a WharfIcon widget.
 */
-IconManager::IconManager(CMainWindow *_mainwin, QPopupMenu *_menu, QWidget *parent)
-  : QWidget(parent, "LicqWharf", WType_TopLevel)
+IconManager::IconManager(CMainWindow *_mainwin, QPopupMenu *_menu, QWidget* /* parent */)
+  : QWidget(NULL, "LicqWharf", WType_TopLevel)
 {
   setCaption("LicqWharf");
   m_nNewMsg = m_nSysMsg = 0;
   wharfIcon = NULL;
   mainwin = _mainwin;
   menu = _menu;
+
+  setBackgroundMode(X11ParentRelative);
 }
 
 
@@ -86,9 +92,14 @@ void IconManager::X11Init()
   WId win = winId();     // get the window
   XWMHints *hints;  // hints
   XClassHint classhint;  // class hints
+  XSizeHints sizeHints;
   classhint.res_name = "licq";  // res_name
   classhint.res_class = "Wharf";  // res_class
   XSetClassHint(dsp, win, &classhint); // set the class hints
+  sizeHints.flags= USPosition;
+  sizeHints.x = 0;
+  sizeHints.y = 0;
+  XSetWMNormalHints(dsp, win, &sizeHints);
   hints = XGetWMHints(dsp, win);  // init hints
   hints->initial_state = WithdrawnState;
   hints->icon_x = 0;
@@ -99,11 +110,19 @@ void IconManager::X11Init()
   XSetWMHints(dsp, win, hints);  // set the window hints for WM to use.
   XFree( hints );
 
+  XMapWindow(dsp, wharfIcon->winId());
+  XMapWindow(dsp, win);
+  XSync(dsp, False);
+
 #endif
 
   resize (wharfIcon->width(), wharfIcon->height());
   setMask(*wharfIcon->vis->mask());
+  move(-100,-100);
   show();
+#ifndef USE_KDE
+  XWithdrawWindow(dsp, win, XScreenNumberOfScreen(DefaultScreenOfDisplay(dsp)));
+#endif
 }
 
 
@@ -200,6 +219,7 @@ void IconManager_Default::SetDockIconStatus()
 {
   QPixmap m;
   ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  if (o == 0) return;
   switch (o->Status())
   {
     case ICQ_STATUS_ONLINE: m = QPixmap((const char **)iconOnline_xpm); break;
@@ -244,6 +264,8 @@ void IconManager_Default::SetDockIconStatus()
 QPixmap *IconManager_Default::GetDockIconStatusIcon()
 {
   ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  if (o == 0) return NULL;
+  
   unsigned long s = o->Status();
   gUserManager.DropOwner();
   switch (s)
@@ -315,7 +337,9 @@ void IconManager_Default::SetDockIconMsg(unsigned short nNewMsg, unsigned short
   {
     // Draw the little icon now
     QPixmap *m = NULL;
-    if (nSysMsg > 0 || nNewMsg > 0)
+    if (nSysMsg > 0)
+      m = &mainwin->pmReqAuthorize;
+    else if (nNewMsg > 0)
       m = &mainwin->pmMessage;
     else
       m = GetDockIconStatusIcon();
@@ -524,6 +548,7 @@ void IconManager_Themed::SetDockIconStatus()
 {
   QPixmap *p = NULL;
   ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  if (o == 0) return;
   switch (o->Status())
   {
     case ICQ_STATUS_ONLINE: p = pixOnline; break;
@@ -642,6 +667,7 @@ void IconManager_KDEStyle::SetDockIconStatus()
 {
   //TODO make general for all plugins
   ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  if (o == 0) return;
   m_statusIcon = CMainWindow::iconForStatus(o->StatusFull());
   m_ownerStatus = o->Status();
   m_bStatusInvisible = o->StatusInvisible();
@@ -658,7 +684,12 @@ void IconManager_KDEStyle::SetDockIconMsg(unsigned short nNewMsg, unsigned short
   m_nSysMsg = nSysMsg;
 
   killTimers();
-  if(m_nNewMsg + m_nSysMsg)
+  if (m_nSysMsg > 0)
+  {
+    m_eventIcon = mainwin->pmReqAuthorize;
+    startTimer(500);
+  }
+  else if (m_nNewMsg > 0)
   {
     m_eventIcon = mainwin->pmMessage;
     startTimer(500);
@@ -671,13 +702,13 @@ void IconManager_KDEStyle::SetDockIconMsg(unsigned short nNewMsg, unsigned short
   repaint();
 }
 
-void IconManager_KDEStyle::timerEvent(QTimerEvent* e)
+void IconManager_KDEStyle::timerEvent(QTimerEvent* /* e */)
 {
   m_timerToggle = !m_timerToggle;
   repaint();
 }
 
-void IconManager_KDEStyle::paintEvent( QPaintEvent *e)
+void IconManager_KDEStyle::paintEvent( QPaintEvent* /* e */)
 {
   QPainter p(this);
 
@@ -693,7 +724,7 @@ bool IconManager_KDEStyle::x11Event(XEvent *ev)
   {
     XWindowAttributes a;
     XGetWindowAttributes(qt_xdisplay(), ev->xreparent.parent, &a);
-    move((a.width - width()) / 2, (a.height - height()) / 2);
+  //  move((a.width - width()) / 2, (a.height - height()) / 2);
   }
   return QWidget::x11Event(ev);
 }
@@ -703,19 +734,19 @@ void IconManager_KDEStyle::mousePressEvent( QMouseEvent *e )
   switch(e->button())
   {
   case LeftButton:
-    if(mainwin->isVisible())
+    if(mainwin->isVisible() && !mainwin->isMinimized())
       mainwin->hide();
     else
     {
       mainwin->show();
 #ifdef USE_KDE
       KWin::setOnDesktop(mainwin->winId(), KWin::currentDesktop());
-  #if !KDE_IS_VERSION(3,1,94) 
-      mainwin->raise();
-  #endif
-#else
-      mainwin->raise();
 #endif
+      if (mainwin->isMaximized())
+      	mainwin->showMaximized();
+      else
+      	mainwin->showNormal();
+      mainwin->raise();
     }
     break;
    default:
@@ -727,8 +758,8 @@ void IconManager_KDEStyle::mousePressEvent( QMouseEvent *e )
 void IconManager_KDEStyle::updateTooltip()
 {
   QToolTip::remove(this);
-  QString s = QString("<nobr>") + QString(ICQUser::StatusToStatusStr(m_ownerStatus, m_bStatusInvisible))
-      + QString("</nobr>");
+  QString s = QString("<nobr>%1</nobr>")
+      .arg(Strings::getStatus(m_ownerStatus, m_bStatusInvisible));
 
   if(m_nSysMsg)
     s += tr("<br><b>%1 system messages</b>").arg(m_nSysMsg);
@@ -755,7 +786,7 @@ WharfIcon::WharfIcon(QPixmap *p, QWidget *parent)
   QToolTip::add(this, tr("Left click - Show main window\n"
                          "Middle click - Show next message\n"
                          "Right click - System menu"));
-  show();
+  //show();
 }
 
 
@@ -765,6 +796,7 @@ void WharfIcon::Set(QPixmap *pix)
   vis = new QPixmap(*pix);
   resize(vis->width(), vis->height());
   setMask(*vis->mask());
+  repaint();
 }
 
 WharfIcon::~WharfIcon()
@@ -787,7 +819,10 @@ void WharfIcon::paintEvent( QPaintEvent * )
 #ifdef DEBUG_WHARF
   printf("wharf paint\n");
 #endif
+  if (!vis) return;
   QPainter painter(this);
   painter.drawPixmap(0, 0, *vis);
   painter.end();
 }
+
+#include "wharf.moc"
